import dns

from dns.resolver import Resolver
import hashlib

def searchMetadata( name, server=None, filter=None, prioritize=True,
                    community=None, environment=None, hashfunc=None):
    r=Resolver()
    
    # if you have a test DNS server running somewhere on your LAN that you want to use
    # instead of the Internet DNS,  specify it here.  Otherwise the environment default
    # DNS server is used
    if server != None:
        r.nameservers = [ server ]

    # It is possible to specify that certain normalizations are to be applied to names
    # This is likely to also depend on community configuration
    if hashfunc == 'md5':
        name = hashlib.md5( name ).hexdigest()
    elif hashfunc == 'sha1':
        name = hashlib.sha1( name ).hexdigest()

    # A community can have a designed domain which serves a suffix for queries
    # By default queries go to the "test" environment, otherwise they go to
    # a named environment
    if community != None:
        if environment != None:
            name = '{}.{}'.format(name,environment)
        else:
            name = '{}.test'.format(name,environment)
        name = '{}.{}'.format(name,community)

    #print "looking up "+name

    # search the DNS server for NAPTR records matching the name
    records  = r.query( name, 'NAPTR' )

    # only use 'U' type NAPTR records
    records = [ item for item in records if item.flags == 'U' ]

    # optionally select only specific service records
    if filter != None:
        records = [ item for item in records if item.service == filter ]

    # optionally sort the records based on priority
    if prioritize:
        records = sorted( records, cmp=lambda x,y: cmp(x.order, y.order ))

    return records


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(description='Perform a metadata lookup')
    parser.add_argument('name', help='Name to look up', type=str )
    parser.add_argument('dnsserver', help='DNS server address', type=str )
    parser.add_argument('--filter', type=str, default=None)
    parser.add_argument('--prioritize', action='store_true'  )
    parser.add_argument('--community', type=str, help='Community or Service Provider Domain' )
    parser.add_argument('--environment', type=str, help='Production or test environment', choices=['production','test'] )
    parser.add_argument('--hashfunc', type=str, choices=['sha1','md5'], help='Function to apply to name')
    p = parser.parse_args()

    records = searchMetadata(p.name, p.dnsserver, p.filter, p.prioritize,
                             p.community, p.environment, p.hashfunc )
    for a in records:
        print a.flags, a.order, a.preference, a.regexp, a.replacement, a.service

